from office365.entity import Entity


class OutlookCategory(Entity):
    """epresents a category by which a user can group Outlook items such as messages and events.
    The user defines categories in a master list, and can apply one or more
    of these user-defined categories to an item."""
    pass
