from office365.sharepoint.fields.field import Field


class FieldDateTime(Field):
    """Specifies a field (2) that contains date and time values. To set properties, call the Update method
    (section 3.2.5.44.2.1.5)."""
