from office365.runtime.client_value import ClientValue


class ChoiceColumn(ClientValue):
    """
    The choiceColumn on a columnDefinition resource indicates that the column's values can be selected
    from a list of choices.
    """
    pass
