class ViewScope:
    """Specifies the scope for returning list items and list folders in a list view."""
    def __init__(self):
        pass

    DefaultValue = "DefaultValue"
    Recursive = "Recursive"
    RecursiveAll = "RecursiveAll"
    FilesOnly = "FilesOnly"
