from generator.templates.entity_type import EntityType


class ProvisioningObjectSummary(EntityType):
    """Represents an action performed by the Azure AD Provisioning service and its associated properties."""
    pass
