class BaseType:
    """Specifies the base type for a list."""

    None_ = -1
    GenericList = 0
    DocumentLibrary = 1
    Unused = 2
    DiscussionBoard = 3
    Survey = 4
    """Enumeration whose values specify a base type for a survey list."""
    Issue = 5
