from office365.runtime.client_value import ClientValue


class SystemFacet(ClientValue):
    """
    The System facet indicates that the object is managed by the system for its own operation.
    Most apps should ignore items that have a System facet.
    """
    pass
