from office365.entity import Entity


class MessageRule(Entity):
    """A rule that applies to messages in the Inbox of a user.

    In Outlook, you can set up rules for incoming messages in the Inbox to carry out specific actions
    upon certain conditions."""
    pass
