from office365.entity import Entity


class ItemAnalytics(Entity):
    """The itemAnalytics resource provides analytics about activities that took place on an item.
    This resource is currently only available on SharePoint and OneDrive for Business."""
    pass
