from office365.outlook.mail.attachment import Attachment


class ReferenceAttachment(Attachment):
    """A link to a file (such as a text file or Word document) on a OneDrive for Business cloud drive or other
    supported storage locations, attached to an event, message, or post."""
    pass
