class ODataMetadataLevel:
    """The amount of metadata information to serialize in an OData response."""
    def __init__(self):
        pass

    Verbose = "verbose"
    NoMetadata = "nometadata"
    MinimalMetadata = "minimalmetadata"
