class PageType:

    def __init__(self):
        pass

    Invalid = -1
    DefaultView = 0
    NormalView = 1
    DialogView = 2
    View = 3
    DisplayForm = 4
    DisplayFormDialog = 5
    EditForm = 6
    EditFormDialog = 7
    NewForm = 8
    NewFormDialog = 9
    SolutionForm = 10
    PAGE_MAXITEMS = 11
